****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_compute_baselinetable.sas  
*
* Created (mm/dd/yyyy): 12/27/2017
* Last modified: 02/27/2019
* Version: 2.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: The macro computes a baseline table
*                                        
*   
*  Program inputs:                                                                                   
*  	-
* 
*  Program outputs:                                                                                                                                       
*  	-
* 
*  PARAMETERS:        
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*	1.1		  07/17/18	   AP		Changed &grp to &grps
*
*	2.0	 	  12/18/18	   AP		Modified to run with both T1/T2 and T5 reporting macros
*
*	2.1	 	  02/27/19	   AP		DEV-4356: Fixed bug in specification of RecordedHistory
*
***************************************************************************************************;

%macro ms_compute_baselinetable();

	%put =====> MACRO CALLED: ms_compute_baselinetable v2.1;

	%let table=tab_agg_l&i.;

	%do grps = 1 %to &num_allgroups.;

		%if "&CREATEREPORT_TYPE" = "1" | "&CREATEREPORT_TYPE" = "2" %then %do;
      	 	%let group = %scan(&allgroups,&grps);
		%end;

		%if "&CREATEREPORT_TYPE" = "5" %then %do;
			data _null_;
				set &groupfile.(where=(order=&grps.));
				if _n_=1 then do;
				call symputx("group" , group);
				call symputx("agegroupfmt", agegroupfmt);
				end;
			run;

			*Remove age groups that are not specified in format;
			%if %str(&agegroupfmt) ne %str() | %length(&agegroupfmt) > 0 %then %do;
				data table1_grp&grps._&i.;
					set table1_grp&grps._&i.;
					if upcase(substr(_NAME_, 1,3)) = 'AGE' then do;
						tempname = translate(substr(_NAME_, 4),'-', '_');
						if index(tempname,'-')=0 then tempname = cats(tempname,'+');
						_label_=strip(put(tempname, $age&grps.fmt.));
						drop tempname;
					end;
					if upcase(substr(_NAME_, 1,3)) = 'AGE' & _label_ = "Other Age" then delete;
				run;
			%end;
		%end;

		/*Put total number of episodes and total number of patients into macro variable*/ 
		/*Type 1/2 report - differs according to cohortdef, Type 5 report - these are equal*/
		data _null_; 
			set table1_grp&grps._&i.(where=(lowcase(_name_)='n_episodes'));     
		    call symputx("totepis", &group.);
		run;
		%put totepis ==> &totepis.;

		data _null_;
			set table1_grp&grps._&i.(where=(lowcase(_name_)='patient'));
			call symputx("totpts",  &group.);
		run;
		%put totpts ==> &totpts.;

		data _percent(keep=_name_ _label_ percent totdic)
			 _mean(keep=_name_ _label_ mean)
			 _std(keep=_name_ std); 

			set table1_grp&grps._&i.;

		    totepis = "&totepis."; /*Total number of episodes in Group*/
			totpts = "&totpts.";   /*Total number of patients in Group*/

			if (index(_name_, 'mean') = 0) and (index(_name_, 'std') = 0) then do;
			   	var_type = 'CATEGORICAL';
				totdic = &group.;
				** Calculate percent;
				if index(upcase(_name_), 'SEX') >0 | index(upcase(_name_), 'RACE') >0 | index(upcase(_name_), 'HISPANIC') >0 then do;
					if ^missing(&group.) and (totpts gt 0) then percent = &group./totpts;
				end;
				else do;
					if ^missing(&group.) and (totepis gt 0) then percent = &group./totepis; 	
				end;
				output _percent;
			end;
	  		else do;
		    	var_type = 'NUMERIC';
		        if (index(_name_, 'mean') > 0) then do;
			        if ^missing(&group.) then mean = &group.;
						_name_= strip(tranwrd(_name_,'mean_',''));
						output _mean;
			    end;
		        if (index(_name_, 'std') > 0) then do;
		          		if ^missing(&group.) then std = &group.;
		            	_name_= strip(tranwrd(_name_, 'std_',''));
						output _std;
		    	end;
			end;
		run;

		/*Create table for each group*/
		proc sort data=_mean; by _name_;
		proc sort data=_std; by _name_;

		data numeric (keep=_name_ _label_ &group._a &group._b vartype); 
			length _label_ $70 &group._a &group._b $15 vartype $15;
			merge _mean _std; by _name_; 
			/*assign mean (std) to group variable*/
			if ^missing(mean) then &group._a = strip(put(mean, comma12.1)); else &group._a = '-';
			if ^missing(std)  then &group._b = strip(put(std, 8.1)); else &group._b = '-';
			/*Define Labels*/
				if upcase(_name_) = 'AGE' then _label_ = 'Mean Age';
				if upcase(_name_) = "NUMCLASS" then _label_ = "Mean number of unique drug classes";
				if upcase(_name_) = "NUMGENERIC" then _label_ = "Mean number of generics";
				if upcase(_name_) = "NUMRX" then _label_ = "Mean number of filled prescriptions";
				if upcase(_name_) = "NUMAV" then _label_ = "Mean number of ambulatory encounters (AV)";
				if upcase(_name_) = "NUMED" then _label_ = "Mean number of emergency room encounters (ED)";
				if upcase(_name_) = "NUMIP" then _label_ = "Mean number of inpatient hospital encounters (IP)";
				if upcase(_name_) = "NUMIS" then _label_ = "Mean number of non-acute institutional encounters (IS)";
				if upcase(_name_) = "NUMOA" then _label_ = "Mean number of other ambulatory encounters (OA)";
				if index(upcase(_name_),'FOLLOWUP') > 0 then _label_ = "Person time at risk";
			vartype = 'continuous';
		run;

		data categorical (keep=_name_ _label_ &group._a &group._b vartype); 
			length _label_ $70 &group._a &group._b $15 vartype $15;
			set _percent;
			/*assign count (percent%) to group variable*/
			if upcase(_name_) not in ('N_EPISODES', 'PATIENT') then do;
				if ^missing(totdic) then do;
					&group._a = strip(put(totdic, comma12.0));
					&group._b = strip(put(100*percent, 8.1)||'%');
				end;
				else do;
					&group._a = '-';
					&group._b = '-';
				end;
			end;
			else do; 
				if ^missing(totdic) then &group._a = compress(put(totdic, comma12.0)); else &group._a = '-';
				&group._b = '';
			end;

			/*Define Labels*/
			if index((upcase(_name_)),'AGE') > 0 then do;
				if _label_ = '' then do;
				if countc(_name_, '_') = 0 then _label_ = 'Age: '||compress((substr(_name_, 4))) ||'+'; 
				else _label_ = 'Age: '||translate(substr(_name_, 4),'-','_'); 
				end;
				else do;
					_label_ = 'Age: '||strip(_label_);
				end;
			end;
			if index((upcase(_name_)),'SEX') > 0 then do;
				if substr(_name_, 5) = 'M' then _label_ = 'Gender (Male)';
				if substr(_name_, 5) = 'F' then _label_ = 'Gender (Female)';
				if substr(_name_, 5) = 'O' then _label_ = 'Gender (Other)';
			end;
			if index((upcase(_name_)),'RACE') > 0 then do;
				if substr(_name_, 6) = '0' then _label_ = 'Race (Unknown)';
				if substr(_name_, 6) = '1' then _label_ = 'Race (American Indian or Alaska Native)';
				if substr(_name_, 6) = '2' then _label_ = 'Race (Asian)';
				if substr(_name_, 6) = '3' then _label_ = 'Race (Black or African American)';
				if substr(_name_, 6) = '4' then _label_ = 'Race (Native Hawaiian or Other Pacific Islander)';
				if substr(_name_, 6) = '5' then _label_ = 'Race (White)';
			end;
			if index((upcase(_name_)),'HISPANIC') > 0 then do;
				if substr(_name_, 10) = 'Y' then _label_ = 'Hispanic Origin';
				if substr(_name_, 10) in ('N', 'U') then delete; 
			end;
			if index((upcase(_name_)),'YEAR') > 0 then do;
				_label_ = 'Year ('||compress(substr(_name_,6))||')';
			end;
			if upcase(_name_) = "N_EPISODES" then _label_ = "Number of episodes";
			if upcase(_name_) = "PATIENT" then _label_ = "Number of unique patients";
			if index(upcase(_name_),'EVENT') > 0 then _label_ = "Number of events while on therapy";
			if index(upcase(_name_),'COVAR') > 0 then _label_ = (tranwrd(_label_,"_", " "));
			vartype = 'dichotomous';
		run;

		data &table._&grps.; 
			retain _name_ _label_ &group._a &group._b; 
			set numeric categorical; 
			label &group._a='N|Mean' 
				  &group._b='%|Std';
		run;
		proc sort data=&table._&grps. sortseq=linguistic(numeric_collation=on); 
			by _name_; 
		run;
				
		proc datasets lib=work nowarn nolist;
			delete _mean _std numeric categorical; 
		quit;

		/* Create covariate macro variables to ensure correct grouping in output*/
		/* By default, all covariates will be grouped under 'Recorded history of' in the output and all utilization vars
		   will be excluded. To exclude covariates, or to include a 'History of Use' section, list covariates below, 
		   separated by a comma. To group covariates, use a dash (i.e. COVAR1-COVAR5, COVAR8, COVAR10-COVAR12
  		   NOTE: if RecordedHistory, HistoryofUse, or UtilizationIntensity is specified, then only covariates explicitly listed will be output*/

			*initialize;
			%let RecordedHistory =  ;
			%let HistoryofUse = ;
			%let UtilizationIntensity= ;
			%let FinalUtilization = ;
			%let FinalHistoryofUse = ;
			%let FinalUtilization = ;

			*assign from &groups_table;
			data _null_;
				set &groupfile.(where=(order=&grps.));

				call symputx('RecordedHistory', upcase(RecordedHistory));
				call symputx('HistoryofUse', upcase(HistoryofUse));
				call symputx('UtilizationIntensity', upcase(UtilizationIntensity));
			run;
					
			%if %length(&RecordedHistory) > 0 or %length(&HistoryofUse) > 0 or %length(&UtilizationIntensity) > 0 %then %do;
					
				%if %length(&RecordedHistory) > 0 %then %do;
					%let countvars = %sysfunc(Countw(%quote(&RecordedHistory), %str(,)));
						%do RHCount = 1 %to &countvars.;
							%let RHWord = %scan(%quote(&RecordedHistory),&RHCount, %str(,));
								%if %index(&RHWord.,%str(-)) = 0 %then %do;
									%if &RHCount = 1 %then %do;
										%let FinalRecordedHistory = "&RHWord.";
									%end;
									%else %let FinalRecordedHistory = &FinalRecordedHistory., "&RHWord.";
								%end;
								%else %do;
									%let start = %sysfunc(substr(%scan(%quote(&RHWord.),1,%str(-)), 6));
									%let end = %sysfunc(substr(%scan(%quote(&RHWord.),2,%str(-)), 6));
									%do covar = &start. %to &end.;
										%if &RHCount = 1 and &covar = &start. %then %do;
											%let FinalRecordedHistory = "COVAR&covar.";
										%end;
										%else %let FinalRecordedHistory = &FinalRecordedHistory. , "COVAR&covar.";
									%end;
								%end;
						%end;
				%end;
				%else %let FinalRecordedHistory = '';

				%if %length(&HistoryofUse) > 0 %then %do;
					%let countvars = %sysfunc(Countw(%quote(&HistoryofUse), %str(,)));
						%do HOUCount = 1 %to &countvars.;
							%let HOUWord = %scan(%quote(&HistoryofUse),&HOUCount, %str(,));
								%if %index(&HOUWord.,%str(-)) = 0 %then %do;
									%if &HOUCount = 1 %then %do;
										%let FinalHistoryofUse = "&HOUWord.";
									%end;
									%else %let FinalHistoryofUse = &FinalHistoryofUse., "&HOUWord.";
								%end;
								%else %do;
									%let start = %sysfunc(substr(%scan(%quote(&HOUWord.),1,%str(-)), 6));
									%let end = %sysfunc(substr(%scan(%quote(&HOUWord.),2,%str(-)), 6));
									%do covar = &start. %to &end.;
										%if &HOUCount = 1 and &covar = &start. %then %do;
											%let FinalHistoryofUse = "COVAR&covar.";
										%end;
										%else %let FinalHistoryofUse = &FinalHistoryofUse. , "COVAR&covar.";
									%end;
								%end;
						%end;
				%end;
				%else %let FinalHistoryofUse = '';

				%if %length(&UtilizationIntensity) > 0 %then %do;
					%let countvars = %sysfunc(Countw(%quote(&UtilizationIntensity), %str(,)));
					%do UtilCount = 1 %to &countvars.;
						%let Utilword = %scan(%quote(&UtilizationIntensity),&UtilCount, %str(,));
							%if &UtilCount = 1 %then %do;
								%let FinalUtilization = "&Utilword.";
							%end;
							%else %let FinalUtilization = &FinalUtilization., "&Utilword.";
					%end;
				%end;
				%else %let FinalUtilization = '';

				%put &FinalRecordedHistory;
				%put &FinalHistoryofUse;
				%put &FinalUtilization;
			%end;

			/*Assign Grouper variable for output and remove event/followup if type 1 analysis*/
			data &table._&grps.; 
				length grouper $60;
				set &table._&grps. (where=(index(upcase(_name_), 'EVENT') = 0 AND index(upcase(_name_), 'FOLLOWUP') = 0
									%if "&CREATEREPORT_TYPE" = "5" %then %do;
									AND index(upcase(_name_), 'EPISODES')=0
									%end;
									));

				sortorder2 = 1; /*for covariates*/
						
				%if "&CREATEREPORT_TYPE" = "1" | "&CREATEREPORT_TYPE" = "2" %then %do;
				if index(upcase(_name_), 'EPISODES') > 0 then do; grouper = 'Characteristic'; sortorder =1; end; else
				%end;

				if upcase(_name_) in ('PATIENT') then do; grouper = 'Characteristic'; sortorder =2; end;
				else if index(upcase(_name_), 'AGE') > 0 then do; 
					if upcase(_name_) ='AGE' then do; sortorder = 4; grouper = 'Patient Characteristics'; end;
					else do; grouper = 'Patient Characteristics'; sortorder =input(cats('5.',_n_), best.); end;
				end;
				else if index((upcase(_name_)),'SEX') > 0 then do; grouper = 'Patient Characteristics'; sortorder =6; end;
				else if index((upcase(_name_)),'RACE') > 0 then do; grouper = 'Patient Characteristics'; sortorder =7; end;
				else if index((upcase(_name_)),'HISPANIC') > 0 then do; grouper = 'Patient Characteristics'; sortorder =8; end;
				else if index((upcase(_name_)),'YEAR') > 0 then do; grouper = 'Patient Characteristics'; sortorder =9; end;


			%if %length(&RecordedHistory.) = 0 and %length(&HistoryofUse.) = 0 and %length(&UtilizationIntensity.) = 0 %then %do;
				else if index(upcase(_name_), 'COVAR') > 0  then do; 
					grouper = 'Recorded history of:'; 
					sortorder =11;
					sortorder2 = input(substr(_name_,6), 8.);
				end;
				else delete;
			%end;
			%else %do;
				else if index(upcase(_name_), 'COMORBIDSCORE') > 0 then do; 
					if 'COMORBIDSCORE' in (&FinalRecordedHistory.) then do;
						grouper = 'Recorded history of:'; 
						_label_ = 'Prior combined comorbidity score'; sortorder =10;
					end;
					else do;
						if 'COMORBIDSCORE' not in (&FinalRecordedHistory.) then delete;
					end;
				end;
				else if upcase(_name_) in (&FinalRecordedHistory.) then do; 
					grouper = 'Recorded history of:'; 
					sortorder =11;
					sortorder2 = input(substr(_name_,6), 8.);
					end;
				else if upcase(_name_) in (&FinalHistoryofUse.) then do; 
					grouper = 'History of use:'; 
					sortorder =12;
					sortorder2 = input(substr(_name_,6), 8.);
				end;
				else if upcase(_name_) in (&FinalUtilization.) then do; 
					grouper = 'Health Service Utilization Intensity:'; 
					if index(upcase(_name_), 'CLASS') or index(upcase(_name_), 'GENERIC') or index(upcase(_name_), 'RX') > 0 then do;
						sortorder = 14;
					end;
					else do;
						sortorder = 13;
					end;
				end;
				else delete;
			%end;
		run;

	%end; /*Looping through groups*/

	%put =====> END MACRO: ms_compute_baselinetable v2.1;

%mend ms_compute_baselinetable;
